/* Copyright 2013 RobustNet Lab, University of Michigan. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.mobiperf_library.api;

import java.security.InvalidParameterException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import com.mobiperf_library.Config;
import com.mobiperf_library.CallbackResult;
import com.mobiperf_library.MeasurementDesc;
import com.mobiperf_library.MeasurementResult;
import com.mobiperf_library.MeasurementTask;
import com.mobiperf_library.measurements.DnsLookupTask;
import com.mobiperf_library.measurements.DnsLookupTask.DnsLookupDesc;
import com.mobiperf_library.measurements.PingTask;
import com.mobiperf_library.measurements.PingTask.PingDesc;
import com.mobiperf_library.measurements.HttpTask;
import com.mobiperf_library.measurements.HttpTask.HttpDesc;
import com.mobiperf_library.measurements.TCPThroughputTask;
import com.mobiperf_library.measurements.TCPThroughputTask.TCPThroughputDesc;
import com.mobiperf_library.measurements.TracerouteTask;
import com.mobiperf_library.measurements.TracerouteTask.TracerouteDesc;
import com.mobiperf_library.measurements.UDPBurstTask;
import com.mobiperf_library.measurements.UDPBurstTask.UDPBurstDesc;

/**
 * @author jackjia,Hongyi Yao (hyyao@umich.edu) The user API for Mobiperf
 *         library User: add task => Scheduler: run task, send finish event =>
 *         User: implement OnResultReturn to get result thread safe The client
 *         and server change TaskID as a Long type.
 */
public class API {
/**
 * Todo(Hongyi): Jack, the following code is not needed and you can
 * just delete it, but I think you can get help from the parameter 
 * checking part
 */
//  private static void parseParam (Map<String, String> params)
//      throws InvalidParameterException {
//    if (params == null) {
//      throw new InvalidParameterException("null parameter map");
//    }
//
//    key = params.get("key");
//    // Todo(Hongyi): how to parse start time and end time?
//    startTime = Calendar.getInstance().getTime();
//    endTime = null;
//
//    try {
//      String val = null;
//
//      if ( (val = params.get("startTime")) != null) {
//        startTime = new Date(Long.parseLong(val));
//        // Todo(Hongyi): add boundary check
//      } else {
//        startTime = Calendar.getInstance().getTime();
//      }
//
//      if ( (val = params.get("endTime")) != null) {
//        endTime = new Date(Long.parseLong(val));
//        // Todo(Hongyi): add boundary check
//      } else {
//        endTime = null;
//      }
//
//      if ( (val = params.get("intervalSec")) != null) {
//        intervalSec = Double.parseDouble(val);
//        if ( intervalSec < Config.MIN_USER_MEASUREMENT_INTERVAL_SEC ) {
//          //intervalSec = Config.DEFAULT_USER_MEASUREMENT_INTERVAL_SEC;
//          throw new InvalidParameterException("Interval second value is too small");
//        }
//        else if ( intervalSec > Config.MAX_USER_MEASUREMENT_INTERVAL_SEC ) {
//          //intervalSec = Config.MAX_USER_MEASUREMENT_INTERVAL_SEC;
//          throw new InvalidParameterException("Interval second value is too big");
//        }
//
//      } else {
//        intervalSec = Config.DEFAULT_USER_MEASUREMENT_INTERVAL_SEC;
//      }
//
//      if ( (val = params.get("count")) != null) {
//        count = Long.parseLong(val);
//        if ( count < Config.MIN_USER_MEASUREMENT_COUNT ) {
//          //count = Config.DEFAULT_USER_MEASUREMENT_COUNT;
//          throw new InvalidParameterException("Run count is too small");
//        }
//        else if ( count > Config.MAX_USER_MEASUREMENT_COUNT ) {
//          //count = Config.MAX_USER_MEASUREMENT_COUNT;
//          throw new InvalidParameterException("Run count is too big");
//        }
//      } else {
//        count = Config.DEFAULT_USER_MEASUREMENT_COUNT;
//      }
//
//      if ( (val = params.get("priority")) != null) {
//        priority = Long.parseLong(val);
//        if ( priority < Config.MIN_USER_MEASUREMENT_PRIORITY ) {
//          //priority = Config.MIN_USER_MEASUREMENT_PRIORITY;
//          throw new InvalidParameterException("Priority is too small");
//        }
//        else if ( priority > Config.MAX_USER_MEASUREMENT_PRIORITY ) {
//          //priority = Config.MAX_USER_MEASUREMENT_PRIORITY;
//          throw new InvalidParameterException("Priority is too big");
//        }
//      } else {
//        priority = Config.DEFAULT_USER_MEASUREMENT_PRIORITY;
//      }
//    } catch (NumberFormatException e) {
//      throw new InvalidParameterException("Task invalid params");
//    }
//  }
	enum TaskType {
		DNSLookup, HTTP, Ping, TCPThroughput, Traceroute, UDPBurst
	}

	private static String key;
	private static Date startTime;
	private static Date endTime;
	private static double intervalSec;
	private static long count;
	private static long priority;

	/**
	 * @author JackJia Params structure used to initialize a DnsLookupTask.
	 */
	private class DnsLookupTaskParams {
		public String target;
		public String server;
	}

	/**
	 * @author JackJia Params structure used to initialize a HttpTask.
	 */
	private class HttpTaskParams {
		public String url;
		public String method;
		public String headers;
		public String body;
	}

	/**
	 * @author JackJia Params structure used to initialize a PingTask.
	 */
	private class PingTaskParams {
		public String target;
		public String packet_size_byte;
		public String ping_timeout_sec;
	}

	/**
	 * @author JackJia Params structure used to initialize a RRCTask.
	 */
	private class RRCTaskParams {
		public String echo_host;
		public String target;
		public String min;
		public String max;
		public String port;
		public String size;
		public String dns;
		public String http;
		public String tcp;
		public String rrc;
		public String result_visibility;
		public String giveup_threshhold;
		public String default_demotion_timers;
	}

	/**
	 * @author JackJia Params structure used to initialize a TCPThroughputTask.
	 */
	private class TCPThroughputTaskParams {
		public String target;
		public String data_limit_mb_down;
		public String data_limit_mb_up;
		public String duration_period_sec;
		public String pkt_size_up_bytes;
		public String sample_period_sec;
		public String slow_start_period_sec;
		public String tcp_timeout_sec;
		public String dir_up;
	}

	/**
	 * @author JackJia Params structure used to initialize a TracerouteTask.
	 */
	private class TracerouteTaskParams {
		public String target;
		public String packet_size_bytes;
		public String ping_timeout_sec;
		public String ping_interval_sec;
		public String ping_per_hop;
		public String max_hop_count;
	}

	/**
	 * @author JackJia Params structure used to initialize a UDPBurstTask.
	 */
	private class UDPBurstTaskParams {
		public String target;
		public String packet_burst;
		public String dest_port;
		public String udp_interval;
		public String direction;
		public String packet_size_byte;

	}

  private static MeasurementTask createTask( TaskParams taskParams) {
    /**
     * TODO(Hongyi): move the following checking into each measurement 
     * tasks' constructor
     */
    //          try {
    //            parseParam(params);
    //          } catch (InvalidParameterException e) {
    //            throw e;
    //          }

    MeasurementTask task;
    MeasurementDesc taskDesc;
    switch ( taskParams.taskType ) {
    case TaskParams.DNSLookup:
      taskDesc = new DnsLookupDesc(
          key, startTime, endTime, intervalSec, count, priority, params );
      task = new DnsLookupTask(taskDesc, getApplicationContext());
      break;
    case TaskParams.HTTP:
      taskDesc = new HttpDesc(
          key, startTime, endTime, intervalSec, count, priority, params );
      task = new HttpTask(taskDesc, getApplicationContext());
      break;
    case TaskParams.Ping:
      taskDesc = new PingDesc(
          key, startTime, endTime, intervalSec, count, priority, params );
      task = new PingTask(taskDesc, getApplicationContext());
      break;
    case TaskParams.TCPThroughput:
      taskDesc = new TCPThroughputDesc(
          key, startTime, endTime, intervalSec, count, priority, params );
      task = new TCPThroughputTask(taskDesc, getApplicationContext());
      break;
    case TaskParams.Traceroute:
      taskDesc = new TracerouteDesc(
          key, startTime, endTime, intervalSec, count, priority, params );
      task = new TracerouteTask(taskDesc, getApplicationContext());
      break;
    case TaskParams.UDPBurst:
      taskDesc = new UDPBurstDesc(
          key, startTime, endTime, intervalSec, count, priority, params );
      task = new UDPBurstTask(taskDesc, getApplicationContext());
      break;
    default:
      throw new InvalidParameterException("Unknown measurement type");
    }
    return task;
  }

  /**
   * @param taskType
   * @param priority
   * @param param
   * @param handler
   * @return
   */
  public static Long addTask ( TaskParams taskParams )
          throws InvalidParameterException {
    // TODO(Hongyi): Is schedeler a object?
    ClientScheduler scheduler = getScheduler();
    if ( scheduler != null ) {
      MeasurementTask task = createTask(taskParams);

      Long taskId = scheduler.addTask(task, taskParams.handler);
      return taskId;
    }
    else {
      // TODO(Hongyi): define no scheduler exception
      return -1L;
    }
  }
  
  public Long addChainingTasks( ArrayList<TaskParams> taskParamsList ) {
    ClientScheduler scheduler = getScheduler();
    if ( scheduler != null ) {
      ArrayList<MeasurementTask> taskList;
      ArrayList<CallbackResult> callbackList;
      for ( TaskParams taskParams : taskParamsList ) {
        taskList.add(createTask(taskParams));
        callbackList.add(taskParams.handler);
      }
      Long taskId = scheduler.addChainingTask(taskList, callbackList);
    }
    else {
      return -1L;
    }
  }

  /**
   * @param task : previous submitted task
   * @return : 
   */
  public static int checkStatus (Long taskId)
      throws InvalidParameterException {
    ClientScheduler scheduler = getScheduler();
    if ( scheduler.isRunning(taskId) ) {
      return RUNNING;
    }
    else if ( scheduler.isInWaitingList(taskId) ) {
      return WAITING;
    }
    else {
      throw new InvalidParameterException("No such taskId");
    }
  }


  /**
   * Pause the task if the task is preemptable, or cancel it
   * if the task is non-preemptable.
   * @param taskId handle to the task
   * @return true for succeed, false for fail
   * @throws InvalidParameterException
   */
  public static boolean pauseMeasurementTask(Long taskId)
      throws InvalidParameterException {
    ClientScheduler scheduler = getScheduler();
    try {
      checkStatus(taskId);
      return scheduler.pause(taskId);
    } catch (InvalidParameterException e) {
      throw e;
    }
  }
  /**
   * Resume the pausing task or start the next task, if the previous task was non-preemptable and have been
   * cancelled.
   * @return return the ongoing task ID after resume. If there is no task to be run, return 0.
   */
  /**
   * Resume the pausing task
   * @param taskId
   * @return true for succeed, false for fail
   * @throws InvalidParameterException
   */
  public static boolean resumeMeasurementTask(Long taskId)
      throws InvalidParameterException {
    ClientScheduler scheduler = getScheduler();
    try {
      checkStatus(taskId);
      return scheduler.resume(taskId);
    } catch (InvalidParameterException e) {
      throw e;
    }

  }

  /**
   * Cancel the task 
   * @param taskId task to be cancelled
   * @return true for succeed, false for fail
   * @throws InvalidParameterException
   */
  public static boolean cancelMeasurementTask(Long taskId)
      throws InvalidParameterException {
    ClientScheduler scheduler = getScheduler();
    try {
      checkStatus(taskId);
      return scheduler.cancel(taskId);
    } catch (InvalidParameterException e) {
      throw e;
    }

  }
}
